#include "CAVERN.h++"
#include <unistd.h>

/*
This is a simple test that doesn't really do anything.
It's main purpose is to give you a source code example of
using the CAVERNsoft API in case there are any doubts
from the documentation.
*/

/* Prototypes for user callbacks for CAVERN Events ********************************/

/// To handle events from channels (like broken connection).
void myChannelCB(CAVERN_irbChannel_c::CAVERN_channelEvent_t event,
		 CAVERN_irbChannel_c *thisChannel,
		 void* userData);

/// To handle events from links (like status of remote requests).
void myLinkCB(CAVERN_irbLink_c::CAVERN_linkEvent_t event,
	      CAVERN_irbLink_c *thisLink,
	      CAVERN_irbLink_c::status_t status,
	      void *userData);

/// To handle events from keys (like new data arriving).
void myKeyCB(CAVERN_irbKey_c::CAVERN_irbKeyEvent_t event,
	     CAVERN_irbKey_c *thisKey,
	     void* userData);

main(int argc, char** argv)
{

	/// Startup CAVERN.
	CAVERN_irb_c *personalIRB = CAVERNInit(&argc, &argv, NULL);
	if (!personalIRB) {
		cvrnMesg(NULL,"CAVERN cannot be started.");
		exit(1);
	}

	CAVERN_irb_c::status_t irbStatus;
	CAVERN_irbId_c remoteIRBId;
	CAVERN_irbChannel_c::status_t retStatus;

	/// If there is an argument on the command line create a channel.
	CAVERN_irbChannel_c *aChannel = personalIRB->createChannel();

	/// And open channel to the remote IRB.
	remoteIRBId.setAddress(argv[1]);
	aChannel->trigger(myChannelCB, NULL);
	aChannel->open(&remoteIRBId,NULL,CAVERN_irbChannel_c::RELIABLE, &retStatus);
	if (retStatus != CAVERN_irbChannel_c::OK) {
		cvrnMesg(NULL,"Channel cannot be open.");
		exit(1);
	}

	cvrnMesg(NULL,"Channel open.");

	CAVERN_irbKeyId_c aKeyId, bKeyId;
	aKeyId.setPath("foo/bar/baz"); aKeyId.setName("marley");
	bKeyId.setPath(NULL); bKeyId.setName("tosh");

	CAVERN_irbKey_c *aKey = personalIRB->define(&aKeyId, NULL, &irbStatus);
	aKey->trigger(myKeyCB,NULL);

	cvrnMesg(NULL,"Define done.");

	CAVERN_linkAttrib_c linkAttribute;
	linkAttribute.setInitialSynchType(CAVERN_linkAttrib_c::NONE);

	CAVERN_irbLink_c *aLink = aChannel->link(aKey, &bKeyId, &linkAttribute);

	if (aLink == NULL) {
		cvrnMesg(NULL,"Cannot form link.");
		exit(1);
	}
	aLink->trigger(myLinkCB,NULL);
	cvrnMesg(NULL,"Link done");

	int sze = 17;
	CAVERN_irbKey_c::status_t status;
	CAVERN_irbLink_c::status_t linkStatus;

	aKey->put("WOW THIS IS COOL",&sze, &status);

	aLink->commitRemote(&linkStatus);
	cvrnMesg(NULL,"Remote commit done.");

	aLink->requestRemote(CAVERN_irbLink_c::DEFAULT, &linkStatus);
	cvrnMesg(NULL,"requestRemote done.");

	delete aLink;
	cvrnMesg(NULL,"Link deleted.");

	aLink = aChannel->link(aKey, &bKeyId, &linkAttribute);
	if (!aLink) {
		cvrnMesg(NULL,"Link re-creation failed.");
		exit(1);
	}
	cvrnMesg(NULL,"Link recreated.");

	char *theData;
	aKey->getDirect(theData, &sze, &status);
	cvrnMesg(NULL,"Get direct done.");

	theData[1] = 'a';
	aKey->releaseDirect(&status);
	cvrnMesg(NULL,"Release direct done.");

	aKey->commit(&status);

	// After direct update do an explicit deliver.
	aKey->deliver(&status);
	cvrnMesg(NULL,"Deliver done.");

	aLink->commitRemote(&linkStatus);

	cvrnMesg(NULL,"CAVERN will shutdown now.");

}

void myChannelCB(CAVERN_irbChannel_c::CAVERN_channelEvent_t event,
		 CAVERN_irbChannel_c *thisChannel,
		 void* userData)
{
	if (event == CAVERN_irbChannel_c::BROKEN_CHANNEL) {
		cvrnMesg(NULL,"CHANNEL BROKEN!");
	}
}

void myLinkCB(CAVERN_irbLink_c::CAVERN_linkEvent_t event,
	      CAVERN_irbLink_c *thisLink,
	      CAVERN_irbLink_c::status_t status,
	      void *userData)
{
	if (event == CAVERN_irbLink_c::REMOTE_REQUEST) {
		switch(status) {
		case CAVERN_irbLink_c::FAILED:
			cvrnMesg(NULL,"Remote request FAILED");
			break;

		case CAVERN_irbLink_c::TIMED_OUT:
			cvrnMesg(NULL,"Remote request TIMED_OUT");
			break;

		case CAVERN_irbLink_c::NO_DOWNLOAD_NEEDED:
			cvrnMesg(NULL,"Remote NO DOWNLOAD NEEDED");
			break;
		}
	} else {
		cvrnMesg(NULL,"Link UNKNOWN EVENT");
	}

}

void myKeyCB(CAVERN_irbKey_c::CAVERN_irbKeyEvent_t event,
	     CAVERN_irbKey_c *thisKey,
	     void* userData)
{
	char *data;
	int size;
	CAVERN_irbKey_c::status_t status;

	cvrnMesg(NULL,"Key event occurred.");

	thisKey->getAutoAlloc(data, &size, &status);
	if (size) {
		cvrnMesg(NULL,"NEW DATA: %d bytes", size);

		// Commit data to persistent store.
		thisKey->commit(&status);
		cvrnMesg(NULL,"Commit done.");
		delete data;
	}
}

