/* Author: Jason Leigh
 * (C) 1997 Electronic Visualization Laboratory - Univ Illinois at Chicago
 */

#ifndef _CAVERN_IRB_MCAST_CHANNEL_C
#define _CAVERN_IRB_MCAST_CHANNEL_C

#include "CAVERN_includes.h++"
#include "CAVERN_irb_c.h++"
#include "CAVERN_irbKey_c.h++"
#include "CAVERN_irbMcastLink_c.h++"

/** IRB Multicast Communications Channel class.
A multicast channel opens up to a multicast group rather than to a remote IRB.
Multicast channels use the standard unreliable multicast protocol.
If you wish to multicast between distantly located remote sites you will either
need your system administrator to set up a multicast tunnel, or you need to
create your own version of a tunnel by forming explict CAVERN_irbChannel_c's and links to
the remote site.
Note: when you open a connection to a multicast group on a machine
you cannot open a connection another multicast group from the same machine.
*/

class CAVERN_irbMcastChannel_c {
public:
	CAVERN_irbMcastChannel_c();

	/// Trigger event types for mcast channels.
	enum CAVERN_mcastChannelEvent_t {
		QOS_DEVIATION
	};


	/// Status codes returned from CAVERN_irbChannel_c methods.
	enum status_t  {
		/// Operation succeeded.
		OK,
		/// Operation failed.
		FAILED,
		/// Used for QoS- operation negotiated a lower QoS.
		NEGOTIATED
	};
	
	/** Open the multicast channel to a multicast group.
	    All IRB programs expected to receive the same packets must
	    listen to the same multicast address and port.

	    @param mcastAddr Specifies a multicast address as a string.
	    Multicast addresses should be of the form 224.x.x.x

	    @param mcastPort Specifies a multicast port.

	    @param CAVERN_qosAttrib_c Specifies the desired
	    QoS. It returns set with the QoS it was able to negotiate.
	    Currently this feature has not been implemented since Nexus
	    does not yet support QoS capabilities.

	    @param status returns OK if got contract;
	    NEGOTIATED if had to negotiate for lower
	    QoS; FAILED if completely failed.
	    */
	void open(char * mcastAddr, unsigned short mcastPort, int mcastTTL,
		  CAVERN_qosAttrib_c* qosAttr,
		  CAVERN_irbMcastChannel_c::status_t *retStatus);

	/** Link a local key to the multicast channel.
	    Note any remote IRBs listening to the same channel with a key
	    that is linked with the SAME NAME will receive data from this key if this key
	    is modified. Conversely if any remote IRB modifies the key
	    this key will also be updated. Hence multicast is like a big pool of memory where
	    everything is shared.
	    */
	CAVERN_irbMcastLink_c * link(CAVERN_irbKey_c* localKey);

	// These should be private but purgeStandardChannelAndItsChannelBuffers
	// in cvrn_comm.c++ needs them.
	enum mcastChannelState_t {
		CLOSED,
		OPEN
	};

	// Use this to prevent the user from opening a channel twice.
	mcastChannelState_t mcastChannelState;

private:
	void (*triggerCallback)(CAVERN_mcastChannelEvent_t event,
			 CAVERN_irbMcastChannel_c *thisChannel,
			 void* userData);

	void *triggerUserData;
	unsigned long associatedMcastChannelId; // Packs both mcastgroup and mcast port

};


#endif
