/* Author: Jason Leigh
 * (C) 1997 Electronic Visualization Laboratory - Univ Illinois at Chicago
 */

#include "CAVERN_includes.h++"

/** Class to define network Quality of Service parameters. QoS attributes allow
you to negotiate how much bandwidth, latency, and jitter is acceptable over
your subscription to a remote IRB.
*/
class CAVERN_qosAttrib_c {

public:
	/** Strategies for acquiring QOS.
	  @see setQosGetRule, getQosGetRule
	  */
	enum CAVERN_qosGetRule_t  {
		/// Get exact QoS desired or fail.
		QOS_GET_EXACT,
		/// Ask for QoS but negotiate for best that can be gotten.
		QOS_NEGOTIATE
	};

	/// Set QoS rule.
	void setQosGetRule(CAVERN_qosGetRule_t);

	/// Set acceptable latency in milliseconds.
	void setAcceptableLatency(float);

	/// Set acceptable jitter in milliseconds.
	void setAcceptableJitter(float);

	/// Set bandwidth in bits per second.
	void setBandwidth(int);

	/// Accessor for QoS rule.
	CAVERN_qosGetRule_t getQosGetRule();

	/// Accessor for latency.
	float getAcceptableLatency();

	/// Accessor for jitter.
	float getAcceptableJitter();

	/// Accessor for bandwidth (in bits per second).
	int getBandwidth();

private:

	CAVERN_qosGetRule_t qosGetRule;
	int bandwidthAllocation;
	float acceptableLatency;
	float acceptableJitter;

};
