#include "CAVERN_includes.h++"

/** CAVERN direct support for multicast.
It is not recommended that you use this
except to connect to clients and servers that do not use IRBs (legacy systems).

Using multicast connections are the simplest. Simply open a connection to a specific
multicast group and port number. Multicast group address must begin with 224.xxx.xxx.xxx
and port numbers should be greater than 6000.

Once open, you can send information to all subscribers or get information by
using put and get respectively.

You may also trigger on any incoming data.

Multicast is an connectionless protocol so you won't
know if a connection has been broken. Since it is an unreliable protocol
you should keep the packets small.
*/
class CAVERN_MCASTKey_c : public CAVERN_key_c {
public:
	/// Status codes for UDP operations.
	enum statusCodes_t  {

		/// Operation succeeded.
		OK = 0,
		/// Operation failed.
		FAILED = -1,
		/// A lower QoS service  was negotiated.
		NEGOTIATED = -2,

		/** For nonblocking socket calls this is the status if
		a read is attempted on a socket with no data available to be read.
		*/
		NON_BLOCKING_NO_DATA_TO_READ = -3
	};

	/// Connect to multicast group.
	statusCodes_t open(char *ip, int port);

	/// Trigger Events for UDP.
	enum CAVERN_MCAST_triggerEvent_t  {
		/// New data has arrived.
		NEW_DATA_ARRIVED
	};

	/** Researve Quality of Service
	@param Desired qos attributes. Also returns with contract.
	@return OK if got contract ; NEGOTIATED if had to negotiate for lower ;
	FAILED if completely failed.
	*/
	statusCodes_t reserveQoS(CAVERN_qosAttrib_c *qos);

	/** Trigger on MCAST events. There is currently only 1 event NEW_DATA_ARRIVED.
	  There is no CONNECTION_CLOSED since MCAST is connectionless.
	 */
	void trigger(void (*callback)(CAVERN_MCAST_triggerEvent_t event,
				      CAVERN_MCASTKey_c *thisKey,
				      void* userData),
			void* userData);

	/** Get data.
	  @return status
	  Returns a status code or else status = UNIX errno.
	  @see statusCodes_t

	  @param buffer to fill with data.
	  @param sizeOfBuffer specifies the number of bytes to put into buffer.
	  It also returns the number of bytes successfully extracted.
	  */
	statusCodes_t get(char * buffer, int *sizeOfBuffer);

	/** Send data.
	  @param buffer contains data to send.
	  @param sizeOfBuffer contains amount of data in buffer to send. Returns
	  the amount of data successfully sent.
	  @return status:
	  Returns a status code or else status = UNIX errno.
	  @see statusCodes_t
	  */
	statusCodes_t put(char *buffer, int *sizeOfBuffer);

	/// Get an incoming message's ip and port.
	//@{
	/// Get incoming message's ip.
	unsigned long getReceiveIP();
	/// Get incoming message's ip as a string.
	void getReceiveIP(char *ip);
	/// Get incoming message's port.
	unsigned short getReceivePort();
	//@}


	/// Get mcast destination's ip and port.
	//@{
	/// Get mcast destination ip.
	unsigned long getSendIP();
	/// Get mcast destination ip as string.
	void getSendIP(char *ip);
	/// Get mcast destination port.
	unsigned short getSendPort();
	//@}

	/// Get this key's ip and port.a
	//@{
	/// Get self ip.
	unsigned long getSelfIP();
	/// Get self ip  as string.
	void getSelfIP(char *ip);
	/// Get self port.
	unsigned short getSelfPort();
	//@}

	/// Close mcast key and tell IRB to unmanage it.
	void close();
};

