#include "CAVERN_includes.h++"


/** Key to support download of WEB documents. By simply filling in the WEB site's
address, path and filename, the IRB will download the data and store it in
the key for your retrieval. Timestamps are also compared to reduce the need to
download redundant data.
 */
class CAVERN_WWWKey_c : public CAVERN_key_c {
public:

	/// Trigger events for WWW key.
	enum CAVERN_triggerWWWEvent_t  {

		/// WEB access timed out but a cached version exists.
		EVENT_TIME_OUT_BUT_CACHE_EXISTS = -1,

		/// WEB timed out but cached version does NOT exist.
		EVENT_TIME_OUT_BUT_CACHE_NOT_EXIST = -2,

		/// Cannot connect to WEB server.
		EVENT_CANNOT_CONNECT = -3,

		/// Using CHECK_BUT_DONT_DOWNLOAD this tells you that the remote version is newer.
		EVENT_REMOTE_VERSION_NEWER = -4,

		/// No download is needed becoz cached version is up-to-date.
		EVENT_NO_DOWNLOAD_NEEDED = -5,

		/** This means your download attempt failed. Your WEB server
		is sending back non HTTP1.0 compliant data.
		*/
		EVENT_UNABLE_TO_GET_LENGTH_OF_WEB_FILE = -6,

		/// Remote file not found.
		EVENT_REMOTE_FILE_NOT_FOUND = -7,

		/// Downloaded successfully.
		EVENT_DOWNLOADED = -8,

		/// For nonblocking calls to Request. @see request.
		EVENT_PROCESSING = -9,
	};

	
	/// Blocking/Non-blocking attribute type.
	enum CAVERN_blocking_t  {
		/// Make an operation blocking.
		BLOCKING,
		/// Make an operation non-blocking (ie return immediately)
		NON_BLOCKING
	};

	/// Used in setTimeOut() to turn off timeouts during WEB download.
	const int NEVER_TIME_OUT = -1;

	/// Determines how the download should proceed. @see setMode.
	enum downloadMode_t  {
		/// Download by comparing timestamps.
		AUTO_LOAD,
		/// Download always regardless of timestamp.
		FORCED_LOAD,
		/// Check the timestamps but don't download.
		CHECK_BUT_DONT_LOAD
	};

	/// Set remote site of download.
	void setRemoteSite(char*);
	/// Accessor for remote site.
	char* getRemoteSite();

	/// Set remote path of download.
	void setRemotePath(char*);

	//// Accessor for remote path.
	char *getRemotePath();

	/// Set remote file to download.
	void setRemoteFile(char*);

	/// Accessor for remote file.
	char *getRemoteFile();

	/** Set timeout period for download of WEB file. Set to NEVER_TIME_OUT
	if no time out is desired.
	*/
	void setTimeOut(int timeout);

	/// Accessor for timeout.
	int getTimeOut();

	/// Set the mode for downloading WEB file.
	void setMode(downloadMode_t mode);

	/// Accessor for mode.
	downloadMode_t getMode();

	/** Request WEB file. If non blocking then this call returns
	  PROCESSING.
	  If it is blocking then it will block until data is gotten or
	  it has failed.
	  */
	CAVERN_triggerWWWEvent_t request(CAVERN_blocking_t blockingType);

	/** Trigger on any events generated by this key.
	  The callback is called with the status code that caused the trigger.
	  */
	void trigger(void (*callback)(CAVERN_triggerWWWEvent_t event,
				      CAVERN_WWWKey_c *thisKey,
				      void* userData),
		     void* userData);

	/// Get downloaded data.
	char *get(int *sze);
};

