#include "CAVERN_includes.h++"

/** Meta info about a key. More fields will be added over time as user's ask
for them.
*/
class CAVERN_irbKeyInfo_c {
public:

	/// Get key id describing the key. This allocates an object which
	/// you should delete when done.
	CAVERN_irbKeyId_c *getKeyId();

	/// Display parameters of key to stdout.
	void display();

	/// Possible types of keys.
	enum CAVERN_keyType_t  {
		/// The basic key to an IRB. @see CAVERN_irbKey_c
		IRB_KEY,
		/// Key for WEB access. @see CAVERN_WWWKey_c
		WWW_KEY,
		/// Key for direct connections over TCP/IP. @see CAVERN_TCPKey_c
		TCP_KEY,
		/// Key for direct connections over UDP. @see CAVERN_UDPKey_c
		UDP_KEY,
		/// Key for direct connections over multicast. @see CAVERN_MCASTKey_c
		MCAST_KEY
	};

	/// Figure out what kind of key do we have.
	CAVERN_keyType_t getKeyType();

	/** Get the subscription if any, else return NULL.
	  DO NOT ALLOW THIS. If they want a handle on the key they need to
	  use define or makeTCP etc... This getKey will screw a lot of things
	  up.
	 Returns a key either of type: CAVERN_irbKey_c,
	 CAVERN_WWWKey_c, CAVERN_TCPKey_c, CAVERN_UDPKey_c, CAVERN_MCASTKey_c

	CAVERN_key_c *getKey();
	*/

	/// Get size of item currently stored in key.
	int getSize();

	/// Get time stamp of current key.
	double getTimeStamp();

	/// Get total size of incoming replacement for key.
	int getIncomingTotalSize();

	/// Get size downloaded so far of incoming replacement for key.
	int getIncomingCurrentSize();

	/// Get time stamp of incoming replacement for key.
	double getIncomingTimeStamp();

	/// Get current QoS attributes.
	CAVERN_qosAttrib_c *getQos();

	/// Get contracted QoS attributes.
	CAVERN_qosAttrib_c getContractedQos();

	/* OBSOLETE: Returns a handle to the subscription if exists, else returns NULL.
	   DISALLOW THIS. Since handles are now envelopes this would generate
	   another envelope. Also it is stupid to implement
	   semantics where it returns something if it has already been
	   subscribed to else NULL since by common sense you'd expect
	   it to give you the subscription. But you can't just be given
	   the subscription because there are attributes that need to be specified
	   with a subscribe() call.
	CAVERN_key_c * getSubscription();
	*/

	/** Returns the number of links connected to this key from remote
	clients.
	*/
	int getNumberOfLinks();

private:
	CAVERN_irbKeyId_c keyId;
	int keyHashID;
	int size;
	int incomingSize;
	int incomingCurrentSize;
	int incomingTotalSize;
	double timeStamp;
	CAVERN_qosAttrib_c currentQos;
	CAVERN_qosAttrib_c contractedQos;

	/*
	// Need to make these friends of the class that populates it
	// with data. That is to be determined.
	void setSize(int);
	void setTimeStamp(double);
	void setIncomingTotalSize(int);
	void setIncomingCurrentSize(int);
	void setIncomingTimeStamp(double);
	*/

	// This is either filled with a pointer to a local subscription or NULL.
	// CAVERN_key_c *theSubscriptionIfAny;

	int numberOfLinksToKey;

};
