#include "CAVERN_includes.h++"


/** CAVERN Queue Base Class.
Do not allocate an object of this type */
class CAVERN_queue_c {

public:

	/* IGNORE THIS Queue items arriving at the key. 
	  @param keyToQueue This only binds to proper CAVERN database keys.
	  It does not allow binding to direct TCP, UDP, MCAST keys.
	  Also does not allow binding to keys subscribed as
	  UPDATE_BY_PARTIAL_REPLICATION
	virtual statusCodes_t Bind(CAVERN_irbKey_c keyToQueue) = 0;
	  */

	/// Clear the queue.
	virtual void clear() = 0;

	/// Peek at the length of queue.
	virtual int peekLength() = 0;

	/** Determine if a queue is empty.
	@return 1 if empty, else returns 0.
	*/
	virtual int isEmpty() = 0;

};

/** CAVERN discrete queue class.
This takes successive discrete data values entering
a key and queues them.

 */
class CAVERN_discreteQueue_c : public CAVERN_queue_c {

public:
	/** Get data from front of queue.
	  @param sze passed as pointer. Returns sze of data in dequeued object.
	  If queue is empty sze returns 0.
	  @return data buffer of queued object. It is the user's responsibility to
	  deallocate the memory when done. Returns NULL if queue is empty.
	  */
	char* get(int *sze);

	/** Get subpart of data from front of queue. The queued item is then completely
	  dequeued.
	  @param startIndex beginning of subsection to extract.
	  @param endIndex end of subsection to extract. If endIndex is NULL it means
	  extract to the end. endIndex must be either NULL or greater than or equal to
	  startIndex.
	  @param sze passed as pointer. Returns sze of data extracted.
	  If queue is empty sze returns 0.
	  @return data buffer of subsection. It is the user's responsibility to
	  deallocate the memory when done.
	  */
	char* get(int startIndex, int endIndex, int *sze);

	/** Peek at queued item without dequeueing it.
	  @see get(int *sze)
	  */
	char* peek(int *sze);

	/** Peek at subpart of queued item without dequeueing it.
	 @see get(int startIndex, int endIndex, int *sze)
	 */
	char* peek(int startIndex, int endIndex, int *sze);

	/// Dequeue an item.
	void deQueue();

	/// Peek at the sze of item at front of queue.
	int peekQueueItemSize();

	/// Clear the queue.
	void clear();

	/// Peek at the length of queue.
	int peekLength();

	/** Determine if a queue is empty.
	@return 1 if empty, else returns 0.
	*/
	int isEmpty();
	
};

/** CAVERN stream class. This takes successive discrete data values entering
a key and queues them so that it can be accessed as a stream.
*/
class CAVERN_stream_c : public CAVERN_queue_c {

public:
	/** Get data from front of queue.
	  @param sze passed as pointer. Pass in the number of bytes you wish to
	  extract. Returns sze of data extracted.
	  If queue is empty sze returns 0.
	  @return data buffer of queued chunk. It is the user's responsibility to
	  deallocate the memory when done. Returns NULL if queue is empty.
	  */
	char* get(int *sze);

	/// Clear the queue.
	virtual void clear();

	/// Peek at the length of queue.
	virtual int peekLength();

	/** Determine if a queue is empty.
	@return 1 if empty, else returns 0.
	*/
	virtual int isEmpty();
	

};



