#include "CAVERN_includes.h++"

class CAVERN_irbLink_c {
public:
	/// Status codes returned from various methods.
	enum statusCodes_t  {
		/// Operation succeeded.
		OK,
		/// Operation failed.
		FAILED,
		/// Used for QoS- operation negotiated a lower QoS.
		NEGOTIATED
	};


	/// Trigger events
	enum CAVERN_linkTriggerEvent_t  {
		/// Broken link due to break of connection.
		BROKEN_LINK,

		/// Triggers when the QoS of the network service begins to deviate.
		QOS_DEVIATION
	};

	/// Deleting this link will unlink between the local and remote keys.
	~CAVERN_irbLink_c();

	/** Researve Quality of Service
	@param CAVERN_qosAttrib_c Specifies the desired
	QoS. It returns set with the QoS it was able to negotiate.
	@return OK if got contract ; NEGOTIATED if had to negotiate for lower ;
	FAILED if completely failed.
	*/
	statusCodes_t  researveQoS(CAVERN_qosAttrib_c *);

	/** Suspend active updates. You can hence use this as a form of flow control
	for incoming data.
	@see resumeActiveUpdates
	*/
	void suspendActiveUpdates();

	/** Resume active updates.
	  @see suspendActiveUpdates
	  */
	void resumeActiveUpdates();

	/** Trigger on events about the link.
	  For example, if a connection breaks the user can be warned.
	  The system will attempt to re-establish the link, but you
	  can override this by first unlinking and then forming a new
	  link with perhaps another IRB.

	Need API calls to get info on the linked keys.
	*/
	void trigger(void (*callback)(CAVERN_linkTriggerEvent_t event,
				      CAVERN_irbLink_c *thisLink,
				      void* userData),
		     void* userData);

	/// Get a handle on the local key bound to this link.
	CAVERN_irbKey_c *getLocalKey();

	/// Get an id on the remote IRB bound to this link.
	CAVERN_irbId_c *getRemoteIRBID();

	/// Get an id on the remote key bound to this link.
	CAVERN_irbKeyId_c *getRemoteKeyID();
};
