#include "CAVERN_includes.h++"

/** Class to specify the characteristics of a link between two keys.
Characteristics that can be specified include:
network reliability, and active/passive updating.
QoS is not defined here. It is set in CAVERN_irbLink_c since it can involve
a process of negotiation.
*/
class CAVERN_linkAttrib_c {
 public:
	/// Network reliability types
	enum CAVERN_networkReliability_t  {
		/// Use reliable protocols- DEFAULT.
		RELIABLE,
		/// Use unreliable protocols.
		UNRELIABLE
	};

	/// Update types
	enum CAVERN_update_t  {
		/// Active update- DEFAULT.
		ACTIVE_UPDATE,
		/// Passive update.
		PASSIVE_UPDATE
	};


	/** Set update type of link. Either active or passive update.
	  For active updates any changes in the data will automatically
	  be forwarded to the end point of the link. Also any change at the
	  end point will be returned.

	  For passive updates only
	  an indication that the current linked data is obsolete is
	  forwarded. It is recommended that for small pieces of data
	  active updates are used and for large data passive updates.
	  */
	void setUpdateType(CAVERN_update_t);


	/** Set the delivery reliability to use. Either choose reliable or
	  unreliable. Reliable is slower but guarantees data arrival.
	  Unreliable is faster but does not guarantee data arrival nor
	  does it guarantee order. It does guarantee that data that arrives,
	  if it arrives is correct data.

	  CAVERN detemines what is the best medium for transmission
	  based on networking capability and available resources.  So
	  e.g. if you choose reliable it will try reliable multicast
	  if there are more than 2 users on the system. Else it
	  chooses unicast. If multicast is unavailable it defaults to
	  broadcast by unicasting to all subscribers.  For unreliable
	  connections a similar approach is made first with unreliable
	  multicast, then with broadcasting via udp.  Note: A
	  multicast or unicast channel may carry data for more than one
	  key.
	  */
	void setNetworkReliability(CAVERN_networkReliability_t);

	/// Get network reliability attribute.
	CAVERN_networkReliability_t getNetworkReliability();


};

