/* Author: Jason Leigh
 * (C) 1997 Electronic Visualization Laboratory - Univ Illinois at Chicago
 */

/*
Simple non-graphic demo program to test CAVERN support template for avatars.
Run the program by typing either:

demoavatar

or

demoavatar site lifetime

where:

site is the IP address of a remote IRB or another running demoavatar program.

lifetime (in seconds) is the amount of time to run this program before quiting.
By setting different times you can watch as avatars appear and disapper.

*/

#include "CAVERN.h++"
#include "CAVERNst_baseAvatar_c.h++"


// User-defined Callback for when a new avatar arrives
void NewAvatarCB(CAVERNst_baseAvatar_c *anAvatar)
{
	cvrnMesg("USER CALLBACK","WELCOME!!! AVATAR: %d ADDED\n", anAvatar->avatarID);
}

// User-defined Callback for when the avatar has changed
void AvatarHelloChangeCB(CAVERNst_baseAvatar_c *anAvatar)
{
	cvrnMesg("USER CALLBACK","AVATAR %d HELLO\n", anAvatar->avatarID);

}

// User-defined callback for when aux data arrives.
void AvatarAuxCB(CAVERNst_baseAvatar_c *anAvatar)
{
	cvrnMesg("USER CALLBACK","AVATAR %d AUX\n", anAvatar->avatarID);
}

void AvatarTrackerCB(CAVERNst_baseAvatar_c *anAvatar)
{
	cvrnMesg("USER CALLBACK","AVATAR %d TRACKER\n", anAvatar->avatarID);

}

// User-defined callback for when the avatar has exited
void ExitAvatarCB(CAVERNst_baseAvatar_c *anAvatar)
{
	cvrnMesg("USER CALLBACK","AVATAR %d EXIT\n", anAvatar->avatarID);
}

#define HELLO_SIZE 256
#define TRACKER_SIZE 64
#define AUX_SIZE 64

main(int argc, char** argv)
{


	/// Startup CAVERN.
	CAVERN_irb_c *personalIRB = CAVERNInit(&argc, &argv, NULL);
	if (!personalIRB) exit(1);

	CAVERNst_baseAvatarManager_c *avatarManager = new CAVERNst_baseAvatarManager_c;
	avatarManager->setNewAvatarCallback(NewAvatarCB);
	avatarManager->setAvatarHelloChangeCallback(AvatarHelloChangeCB);
	avatarManager->setAvatarAuxCallback(AvatarAuxCB);
	avatarManager->setAvatarTrackerCallback(AvatarTrackerCB);
	avatarManager->setAvatarExitCallback(ExitAvatarCB);

	CAVERN_irbId_c remoteAvatarServerIRB;
	if (argc > 1) {
		remoteAvatarServerIRB.setAddress(argv[1]);
		avatarManager->init("myApp",personalIRB, &remoteAvatarServerIRB,
				    TRACKER_SIZE, HELLO_SIZE, AUX_SIZE);
	} else {
		avatarManager->init("myApp",personalIRB, NULL,
				    TRACKER_SIZE, HELLO_SIZE, AUX_SIZE);
	}

	/// Pack hello data with data about your avatar and then send it.
	char helloData[HELLO_SIZE];
	sprintf(helloData,"THIS IS HELLO DATA");
	avatarManager->setHelloData(helloData);
	avatarManager->sendHello();

	char trackerData[TRACKER_SIZE];
	char auxData[AUX_SIZE];

	int loops;
	if (argc > 2) loops = atoi(argv[2]);
	else loops = 1000;

	for (int i = 0; i < loops ; i++) {

		/// Pack tracker data with whatever data you want and then send it.
		avatarManager->setTrackerData(trackerData);
		avatarManager->sendTracker();
		sleep(2);

		if ((i % 4) == 0){
			sprintf(auxData,"AUX DAT %d",i);
			avatarManager->setAuxData(auxData);
			avatarManager->sendAux();
		}
	}

	avatarManager->sendBye();
}
