#include "lim_global.h++"
#include "lim_perf.h++"
#include <unistd.h>

CAVERNst_baseAvatarManager_c *g_limAvatarManager;
pfDCS *g_limAvatarDCS = NULL;
pfNode *g_limAvatarGenericHead = NULL;
pfNode *g_limAvatarGenericHand = NULL;
pfNode *g_limAvatarGenericBody = NULL;
CAVERNst_baseAvatar_c *g_newAvatarInTown = NULL;

#define _LIM_HELLO_SIZE 256
#define _LIM_TRACKER_SIZE 64
#define _LIM_AUX_SIZE 64

// User-defined Callback for when a new avatar arrives
void NewAvatarCB(CAVERNst_baseAvatar_c *anAvatar)
{

	// If avatar already waiting to be loaded, then wait.
	while(g_newAvatarInTown) sginap(3);

	cvrnMesg("USER CALLBACK","WELCOME!!! AVATAR: %d ADDED\n", anAvatar->avatarID);

	g_newAvatarInTown = anAvatar;

	// Wait till this avatar has been loaded.
	while(g_newAvatarInTown) sginap(3);
}

// User-defined Callback for when the avatar has changed
void AvatarHelloChangeCB(CAVERNst_baseAvatar_c *anAvatar)
{
	cvrnMesg("USER CALLBACK","AVATAR %d HELLO\n", anAvatar->avatarID);

}

// User-defined callback for when aux data arrives.
void AvatarAuxCB(CAVERNst_baseAvatar_c *anAvatar)
{
	cvrnMesg("USER CALLBACK","AVATAR %d AUX\n", anAvatar->avatarID);
}


void addAvatarMainDCS() {
	// Create DCS underwhich all avatars will reside
	g_limAvatarDCS = new pfDCS;
	g_limMainGroup->addChild(g_limAvatarDCS);

	g_limAvatarGenericHead = pfdLoadFile("LIMBO_MODELS/head.iv");
	g_limAvatarGenericHand = pfdLoadFile("LIMBO_MODELS/hand.iv");
	g_limAvatarGenericBody = pfdLoadFile("LIMBO_MODELS/body.iv");
}

void packAndSendSelfAvatarData(CAVERNst_baseAvatarManager_c *manager,
				       float bodyX, float bodyY, float bodyZ,
				       float bodyAngleY,
				       float headAngleX, float headAngleY, float headAngleZ,
				       float handX, float handY, float handZ,
				       float handAngleX, float handAngleY, float handAngleZ)
{

	CAVERNst_baseAvatar_c *selfAvatar = manager->selfAvatar;

	if (!selfAvatar) return;

	selfAvatar->bodyX = bodyX;
	selfAvatar->bodyY = bodyY;
	selfAvatar->bodyZ = bodyZ;
	selfAvatar->bodyAngleY = bodyAngleY;
	selfAvatar->headAngleX = headAngleX;
	selfAvatar->headAngleY = headAngleY;
	selfAvatar->headAngleZ = headAngleZ;
	selfAvatar->handX = handX;
	selfAvatar->handY = handY;
	selfAvatar->handZ = handZ;
	selfAvatar->handAngleX = handAngleX;
	selfAvatar->handAngleY = handAngleY;
	selfAvatar->handAngleZ = handAngleZ;

	CAVERNplus_datapack_c datapack;

	datapack.setBuffer(selfAvatar->trackerData, _LIM_TRACKER_SIZE);
	datapack.packFloat(bodyX);
	datapack.packFloat(bodyY);
	datapack.packFloat(bodyZ);
	datapack.packFloat(bodyAngleY);
	datapack.packFloat(headAngleX);
	datapack.packFloat(headAngleY);
	datapack.packFloat(headAngleZ);
	datapack.packFloat(handX);
	datapack.packFloat(handY);
	datapack.packFloat(handZ);
	datapack.packFloat(handAngleX);
	datapack.packFloat(handAngleY);
	datapack.packFloat(handAngleZ);

	manager->sendTracker();
}

void unpackTrackerData(CAVERNst_baseAvatar_c *avatar)
{
	avatar->lock();
	CAVERNplus_datapack_c datapack;
	datapack.setBuffer(avatar->trackerData, _LIM_TRACKER_SIZE);
	avatar->bodyX = datapack.unpackFloat();
	avatar->bodyY =	datapack.unpackFloat();
	avatar->bodyZ = datapack.unpackFloat();
	avatar->bodyAngleY = datapack.unpackFloat();
	avatar->headAngleX = datapack.unpackFloat();
	avatar->headAngleY = datapack.unpackFloat();
	avatar->headAngleZ = datapack.unpackFloat();
	avatar->handX = datapack.unpackFloat();
	avatar->handY = datapack.unpackFloat();
	avatar->handZ = datapack.unpackFloat();
	avatar->handAngleX = datapack.unpackFloat();
	avatar->handAngleY = datapack.unpackFloat();
	avatar->handAngleZ = datapack.unpackFloat();
	avatar->unlock();


}

void removePerformerAvatar(CAVERNst_baseAvatar_c *aPerson)
{
	if (!aPerson) return;
	pfDCS *avatarDCS = (pfDCS *) aPerson->avatarDCS;

	if (avatarDCS) {
	    cvrnPrintf("Removing avatar\n");
	    if (g_limAvatarDCS) {
		    g_limAvatarDCS->removeChild(avatarDCS);
		    cvrnPrintf("DONE Removing avatar\n");
	    } else {
		    cvrnPrintf("CAN'T REMOVE avatar\n");
	    }
	}


}

void updatePerformerAvatar(CAVERNst_baseAvatar_c *aPerson)
{
	unpackTrackerData(aPerson);

	pfDCS *bodyDCS, *headDCS, *handDCS;
	//    pfDCS *avatarDCS;

	if (aPerson == NULL){
		fprintf(stderr, "updatePerformerAvatar:can't update a NULL avatar\n");
		return;
	}

	aPerson->lock();

	bodyDCS	= (pfDCS *) aPerson->bodyDCS;
	headDCS	= (pfDCS *) aPerson->headDCS;
	handDCS	= (pfDCS *) aPerson->handDCS;
	//    avatarDCS	= (pfDCS *) aPerson->avatarDCS;

	if ((bodyDCS == NULL) || (headDCS == NULL) || (handDCS == NULL)) {
		fprintf(stderr, "updatePerformerAvatar:avatar is missing some body parts\n");
		aPerson->unlock();
		return;
	}

	bodyDCS->setTrans(aPerson->bodyX, aPerson->bodyY, aPerson->bodyZ);
	bodyDCS->setRot(aPerson->bodyAngleY, 0, 0);
	headDCS->setRot(aPerson->headAngleX,aPerson->headAngleY,aPerson->headAngleZ);
	handDCS->setTrans(aPerson->handX, aPerson->handY, aPerson->handZ);
	handDCS->setRot(aPerson->handAngleX,aPerson->handAngleY,aPerson->handAngleZ);
	aPerson->unlock();

}


void limAvatarInit(char ** argv)
{


	g_limAvatarManager = new CAVERNst_baseAvatarManager_c;
	g_limAvatarManager->setNewAvatarCallback(NewAvatarCB);
	g_limAvatarManager->setAvatarHelloChangeCallback(AvatarHelloChangeCB);
	g_limAvatarManager->setAvatarAuxCallback(AvatarAuxCB);
	g_limAvatarManager->setAvatarTrackerCallback(updatePerformerAvatar);
	g_limAvatarManager->setAvatarExitCallback(removePerformerAvatar);

	CAVERN_irbId_c remoteAvatarServerIRB;
	if (g_limRemoteConnect) {
		remoteAvatarServerIRB.setAddress(argv[1]);
		g_limAvatarManager->init("limbo",g_limPIRB, &remoteAvatarServerIRB,
				    _LIM_TRACKER_SIZE, _LIM_HELLO_SIZE, _LIM_AUX_SIZE);
	} else {
		g_limAvatarManager->init("limbo",g_limPIRB, NULL,
				    _LIM_TRACKER_SIZE, _LIM_HELLO_SIZE, _LIM_AUX_SIZE);
	}

	// Send off my name which is name@hostname
	// This name will appear under my tshirt
	char helloData[_LIM_HELLO_SIZE];
	char hostname[_LIM_HELLO_SIZE];
	gethostname(hostname,_LIM_HELLO_SIZE);
	if (g_limRemoteConnect)
		sprintf(helloData,"%s at %s",argv[2],hostname);
	else
		sprintf(helloData,"%s at %s",argv[1],hostname);

	g_limAvatarManager->setHelloData(helloData);
	g_limAvatarManager->sendHello();


}

void limAvatarExit()
{
	g_limAvatarManager->sendBye();

}

void limSendSelfAvatarTracker()
{
	float head[3], headAngle[3], hd[3];
	float wand[3], wandAngle[3], wd[3];


	CAVEGetPosition(CAVE_HEAD,hd);
	CAVENavConvertCAVEToWorld(hd,head);	

	CAVEGetOrientation(CAVE_HEAD,headAngle);

	CAVEGetPosition(CAVE_WAND,wd);
	CAVENavConvertCAVEToWorld(wd,wand);	

	CAVEGetOrientation(CAVE_WAND_NAV,wandAngle);

	packAndSendSelfAvatarData(g_limAvatarManager,
			       head[0],                          head[1],      head[2],
			       -g_limCurrentCaveTurn,
			       headAngle[2],                     headAngle[0], headAngle[1],
			       wand[0],                          wand[1],      wand[2],
			       wandAngle[2], wandAngle[0], wandAngle[1]);
}

void limCheckForNewAvatars()
{
	if (!g_newAvatarInTown) return;

	CAVERNst_baseAvatar_c *anAvatar = g_newAvatarInTown;

	pfDCS *avatarDCS =  new pfDCS;
	pfDCS *bodyDCS = new pfDCS;
	pfDCS *headDCS = new pfDCS;
	pfDCS *handDCS = new pfDCS;

	pfFont      *fnt;
	pfFilePath("./:/usr/share/Performer/data");
	fnt = pfdLoadFont_type1("Times-Elfin", PFDFONT_FILLED);
	if (fnt) {
		pfText * text = new pfText;
		pfDCS * textDCS = new pfDCS;
		bodyDCS->addChild(textDCS);
		textDCS->addChild(text);
		
		textDCS->setTrans(0.0f, 1.0f, -1.2f);
		textDCS->setRot(180.0f, 0.0f,  0.0f);
		textDCS->setScale(0.4f, 0.4f,  0.4f);
	    
		pfString * str = new pfString;
		str->setFont(fnt);
		str->setMode(PFSTR_JUSTIFY,  PFSTR_MIDDLE);
		str->setColor(1.0f, 1.0f, 0.5f, 1.0f);
	    
		str->setString(anAvatar->helloData);
		str->flatten();
		text->addString(str);
	}


	headDCS->addChild(g_limAvatarGenericHead->clone(0));
	handDCS->addChild(g_limAvatarGenericHand->clone(0));
	bodyDCS->addChild(g_limAvatarGenericBody->clone(0));

	bodyDCS->addChild(headDCS);
	avatarDCS->addChild(bodyDCS);
	avatarDCS->addChild(handDCS);

	anAvatar->avatarDCS = (void*) avatarDCS;
	anAvatar->handDCS = (void*) handDCS;
	anAvatar->headDCS = (void*) headDCS;
	anAvatar->bodyDCS = (void*) bodyDCS;

	g_limAvatarDCS->addChild(avatarDCS);

	g_newAvatarInTown = NULL;
}
