/*

LIMBO ZERO

(C) 1998 Jason Leigh, Electronic Visualization Laboratory,
Univ Illinois at Chicago

*/

#include "lim_global.h++"
#include "lim_perf.h++"

pfDCS *g_limMainGroup = NULL;
pfScene *g_limScene = NULL;
pfDCS *g_limWandDCS = NULL;

pfDCS * createScene(pfChannel *chan, pfScene *scene);
void loadMainSceneObjects(pfDCS *parent);

// Initialize performer.
void limPerformerInit(int *argc, char *** argv)
{

	CAVEFar = 100000;
	// Initialize Performer
	pfInit();

	// Equivalent to CAVE Config. Must be called after pfInit()
	pfCAVEConfig(argc,*argv,NULL);

	// Performer config
	pfConfig();

	// Craetes all the channels for performer
	pfCAVEInitChannels();

	// Create a scene composed of objects loaded from filenames specified
	// in the command line arguments.

	g_limScene = new pfScene;
	g_limMainGroup = createScene(pfCAVEMasterChan(),g_limScene);

}

// Perform one frame update
void limPerfOneCycle()
{
		// Required by performer to synch things to a certain frame rate.
      		pfSync();

		// Updates tracker data and sets up new projects for each channel.
		pfCAVEPreFrame();

		// Required by performer to trigger cull and draw
		pfFrame();

		// Updates non latency critical data.
		pfCAVEPostFrame();

}

pfDCS * createScene(pfChannel *chan, pfScene *scene)
{
	/* Create a pfScene */
	pfGeoState *gstate = new pfGeoState;
	pfDCS *group = new pfDCS;

	/* Set up the default GeoState to specify the how the geometry is to
	   be displayed.
	   */
	gstate->setMode(PFSTATE_ENLIGHTING, PF_ON);
	gstate->setMode(PFSTATE_CULLFACE, PFCF_OFF);
	scene->setGState(gstate);

	/* Add a light source to the scene as well as the dcs */
	pfLightSource *myLightSource = new pfLightSource;
	myLightSource->setPos(10,-10,10,1);

	pfLightSource *myLightSource2 = new pfLightSource;
	myLightSource2->setPos(-10,-10,10,1);


	scene->addChild(myLightSource);
	scene->addChild(myLightSource2);


	/* Add the group, and load the objects under it */
	scene->addChild(group);

	loadMainSceneObjects(group);

	/* Assign the scene to the display channel */
	chan->setScene(scene);

	return group;
}

// Load the floor, the limbo logo and the wand pointer.
void loadMainSceneObjects(pfDCS *parent)
{
	pfNode *obj;


	if (obj = pfdLoadFile("LIMBO_MODELS/twirl.iv")) {
		parent->addChild(obj);
	}

	if (obj = pfdLoadFile("LIMBO_MODELS/limbo.iv")) {
              	pfDCS *limboDCS = new pfDCS;
		limboDCS->addChild(obj);
		parent->addChild(limboDCS);
	}

	if (obj = pfdLoadFile("LIMBO_MODELS/wand.iv")) {
		g_limWandDCS = new pfDCS;
		g_limWandDCS->setTravMask(PFTRAV_ISECT,
				  // Intersection mask
				  0x0,
				  PFTRAV_SELF | PFTRAV_DESCEND | PFTRAV_IS_CACHE,
				  PF_OR);

		g_limWandDCS->addChild(obj);

		parent->addChild(g_limWandDCS);
	}

}


// Draw the local user's wand pointer.
void limUpdateWand()
{
	float wandP[3], wandO[3];

	pfCAVEGetPosition(CAVE_WAND_NAV, wandP);
	pfCAVEGetOrientation(CAVE_WAND_NAV,wandO);
	g_limWandDCS->setTrans(wandP[0], wandP[1], wandP[2]);
	g_limWandDCS->setRot(wandO[2],wandO[0],wandO[1]);

}
