#include "CAVERN.h++"

main(int argc, char** argv)
{

	if (argc != 5) {
		fprintf(stderr,"%s remoteIP remotePath remoteName localFileName\n",
			argv[0]);
		exit(1);
	}

	CAVERN_irb_c::status_t irbStatus;
	CAVERN_irbId_c remoteIRBId;
	CAVERN_irbChannel_c::status_t channelRetStatus;
	CAVERN_irbKey_c::status_t keyStatus;


	CAVERN_irb_c *personalIRB = CAVERNInit(&argc, &argv, NULL);
	if (!personalIRB) exit(1);

	/// Create a channel over which communication will occur.
	CAVERN_irbChannel_c *aChannel = personalIRB->createChannel();
	if (aChannel == NULL) {
		cvrnPrintf("Unable to create channel\n");
		exit(1);
	}

	/// Open the channel to a remote IRB specified on the command line.
	remoteIRBId.setAddress(argv[1]);
	aChannel->open(&remoteIRBId,NULL,CAVERN_irbChannel_c::RELIABLE, &channelRetStatus);

	/// Define the local key (called LOCAL_A_KEY)
	CAVERN_irbKeyId_c aKeyId, remote_aKeyId;
	aKeyId.setName("DBGET_KEY");
	if (strcmp(argv[2],"/"))
	remote_aKeyId.setPath(argv[2]);
	remote_aKeyId.setName(argv[3]);

	CAVERN_irbKey_c *aKey = personalIRB->define(&aKeyId, NULL, &irbStatus);

	/// Create a link to link the local key with the remote key.
	CAVERN_linkAttrib_c linkAttribute;
	linkAttribute.setInitialSynchType(CAVERN_linkAttrib_c::NONE);
	//	linkAttribute.setSubsequentSynchType(CAVERN_linkAttrib_c::FORCE_LOCAL_TO_REMOTE);
	CAVERN_irbLink_c *aLink = aChannel->link(aKey, &remote_aKeyId, &linkAttribute);

	if (aLink == NULL) {
		cvrnPrintf("Unable to create link\n");
		exit(1);
	}


	CAVERN_irbLink_c::status_t linkStatus;

	cvrnPrintf("REMOTE REQUESTING...\n");
	aLink->requestRemote(CAVERN_irbLink_c::DEFAULT, &linkStatus, CAVERN_irbLink_c::BLOCKING);
	if (linkStatus == CAVERN_irbLink_c::NO_DOWNLOAD_NEEDED) {
		cvrnPrintf("NO DOWNLOAD NEEDED\n");
		aKey->export(argv[4], &keyStatus);
			if (keyStatus == CAVERN_irbKey_c::OK)
				cvrnPrintf("DONE\n");
			else
				cvrnPrintf("WRITE FAILED\n");
	}
	else
		if (linkStatus == CAVERN_irbLink_c::OK){
			aKey->export(argv[4], &keyStatus);
			if (keyStatus == CAVERN_irbKey_c::OK)
				cvrnPrintf("DONE\n");
			else
				cvrnPrintf("WRITE FAILED\n");
		} else {
			cvrnPrintf("DOWNLOAD FAILED\n");
		}


	delete personalIRB;
}
