#include "lim_global.h++";
#include "kFlatRadar.h++"

/************************************************************************/
/* Performer type data
*/

pfType *kFlatRadar::classType_ = NULL; 

void kFlatRadar::init(void)
{
 if (classType_ == NULL)
	{
        pfDCS::init();				/* Initialize the parent class */
        classType_ =  new pfType(pfDCS::getClassType(), "kFlatRadar");	/* Create the new pfType for this class, based on the parent class's type */
	}
}
/************************************************************************/



kFlatRadar::kFlatRadar()
{
  char objName[10];
  sprintf(objName,"Radar %d", 0);
  cvrnPrintf("%s\n", objName);
  setName(objName);
  setTravMask(PFTRAV_ISECT,
	      // Intersection mask
	      0x8,
	      PFTRAV_SELF | PFTRAV_DESCEND | PFTRAV_IS_CACHE,
	      PF_SET);

 pfGeode *radar;
 pfGeode *you;
 pfVec4 color;
 blips = new pfGroup;
 setType(classType_);  /* set the Performer type of this instance */
 active_ = 1;
 setScale(0.5f);
 prevTime_ = pfGetTime();
 color[0]= 0.00f;
 color[1]= 0.00f;
 color[2]= 0.00f;
 color[3]= 0.3f;
 radar = createCircle(1.0f, color,PFSTATE_TRANSPARENCY); 
 addChild(radar);
 color[0]= 1.0f;
 color[1]= 0.0f;
 color[2]= 0.0f;
 color[3]= 1.0f;
 you = createCircle(0.03f, color,PFSTATE_ENHIGHLIGHTING); 
 addChild(you);
 addChild(blips);
 setTrans(-1.5, 1.5, 5.75);
}

pfGeode * kFlatRadar::createCircle(float radius,pfVec4 color, int mode)
{
 pfGeoSet *gset;
 pfGeoState *gstate;
 pfVec4 *colorAttr;
 pfMatrix matrix;
 pfGeode *geode = new pfGeode;
 active_ = 1;
 gset = pfdNewCircle(16,pfGetSharedArena());

 gstate = new pfGeoState;
 gstate->setMode(mode, 1);

 gset->setGState(gstate);

 colorAttr = (pfVec4 *) pfMalloc(sizeof(pfVec4),pfGetSharedArena());
 *colorAttr = color;
 gset->setAttr(PFGS_COLOR4,PFGS_OVERALL,colorAttr,NULL);
 gset->setDrawMode(PFGS_FLATSHADE, 1);
 matrix.makeScale(radius,radius,radius);
 pfdXformGSet(gset,matrix);
 geode->addGSet(gset);
 return geode;
}

void kFlatRadar::setWorld(pfNode *world)
{
 world_ = world;
}


int kFlatRadar::app(pfTraverser *trav)
{
 int avatarNum, blipsNum;
 pfVec3 myLocation, difference, avatarPos;
 pfDCS *avatar;
 const char *DCSName;
 long avatarID;
 CAVERNst_baseAvatar_c *theAvatar;
 if (!active_)
	return pfDCS::app(trav);
 blipsNum =blips->getNumChildren();
 avatarNum = g_limAvatarDCS->getNumChildren(); 
 for (int i=0; i < blipsNum; i++)
 {
    blips->removeChild((pfDCS*)blips->getChild(0));
 }
 for (i=0; i < avatarNum; i++)
 {
   avatar = (pfDCS*)g_limAvatarDCS->getChild(i);
   DCSName = avatar->getName();
   avatarID = atol(DCSName);
   g_limAvatarManager->avatarDB->find(avatarID, theAvatar);
   CAVEGetPosition(CAVE_HEAD, myLocation.vec);
   avatarPos[0] = theAvatar->bodyX;
   avatarPos[1] = theAvatar->bodyY;
   avatarPos[2] = 0;
   CAVENavConvertWorldToCAVE(avatarPos.vec, difference.vec);
   if ((difference[0] > -75) && (difference[0] < 75) && 
       (difference[1] > -75) && (difference[1] < 75))
   {
      difference[0] = difference[0]/100.0f;
      difference[1] =  difference[1]/100.0f;
      difference[2] = 0.0f;
      pfDCS *blipDCS;
      pfVec4 color;
      pfGeode *blipGeode;
      blipDCS = new pfDCS;
      color[0]= 0.0f;
      color[1]= 1.0f;
      color[2]= 0.0f;
      color[3]= 1.0f;
      blipGeode = createCircle(0.05f, color,PFSTATE_ENHIGHLIGHTING); 
      blipDCS->addChild(blipGeode);
      blips->addChild(blipDCS);
      blipDCS->setTrans(difference[0], difference[1], difference[2]);
      
   }
 }

 return pfDCS::app(trav);
}

int kFlatRadar::needsApp(void)
{
 return TRUE;
}


int kFlatRadar::isActive()
{
    return active_;
}
