/*********************************************************/
// FILENAME:  kPartList.c++
// AUTHORS:   KEVIN CURRY & KENT SWARTZ
// PURPOSE:   Creates a Participant's List that indicates
//            who is sharing the environment
/*********************************************************/


#include <math.h>
#include "lim_global.h++";
#include "kPartList.h++"
#include <Performer/pf/pfText.h>
#include <Performer/pr/pfString.h>

#define Y_OFF -1.0
#define RAY_LENGTH 1000

/************************************************************************/
/* Performer type data*/

pfType *kPartList::classType_ = NULL; 

void kPartList::init(void)
{
 if (classType_ == NULL)
   {
     pfDCS::init(); /* Initialize the parent class */

     // Create the new pfType for this class, based on the parent class's type
     classType_ =  new pfType(pfDCS::getClassType(), "kPartList");	
   }
}
/************************************************************************/



//**************************************************************
//  Function:   CONSTRUCTOR
//  Purpose:    Instantiates the Participant List object
//  Parameters: NONE
//  Returns:    void
//**************************************************************

kPartList::kPartList()
{
  num_collaborators = 0;     // initialize counter

  // Next 2 lines load a font file from the named path
  ////////////////////////////////////////////////////
  pfFilePath("./:/usr/share/Performer/data");
  fnt = pfdLoadFont_type1("Times-Elfin", PFDFONT_FILLED);

  active_ = 1;     // initialize the flag

  titleDCS = new pfDCS();     // instantiate a new DCS to hold titleString

  //set the name for this DCS
  ////////////////////////////////////
  char titleName[10];
  sprintf(titleName,"Title ");
  cvrnPrintf("%s\n", titleName);
  titleDCS->setName(titleName);

  // Next 9 lines set up the name and 
  // intersection mask for this object
  ////////////////////////////////////
  char objName[10];
  sprintf(objName,"List %d", 1);
  cvrnPrintf("%s\n", objName);
  setName(objName);
  setTravMask(PFTRAV_ISECT,
	      // Intersection mask
	      0x8,
	      PFTRAV_SELF | PFTRAV_DESCEND | PFTRAV_IS_CACHE,
	      PF_SET);
  
  titleText = new pfText;     // instantiate a new pfText obj
                              // to attach the titleString to

  // Next 6 lines create and format the string
  //////////////////////////////////////////// 
  pfString* titleString = new pfString;     // instantiate a new pfString
  titleString->setFont(fnt);         // FONT
  titleString->setMode(PFSTR_JUSTIFY, PFSTR_MIDDLE);     // ALIGNMENT
  titleString->setColor(0.0f, 0.0f, 1.0f, 1.0f);     // GREEN
  titleString->setString("Participant List");     // WRITTEN TEXT  
  titleString->flatten();     //  flatten the string

  titleText->addString(titleString);  //  add the pfString to the pfText
  titleDCS->addChild(titleText);     // add the pfText to the pfDCS
  addChild(titleDCS);     // add the pfDCS to this object's DCS
  setScale(0.2f, 0.2f, 0.2f);     // resize this object's  DCS
}


//**************************************************************
//  Function:   getDistance
//  Purpose:    calculates distance between you and user @
//              'avatarLocation'
//  Parameters: your location, other person's location
//  Returns:    distance as a double
//**************************************************************
double kPartList::getDistance(pfVec3 myLocation,pfVec3 avatarLocation)
{
  // this is just your average distance function
  return sqrt(pow(avatarLocation[0] - myLocation[0], 2) +  
              pow(avatarLocation[1] - myLocation[1], 2) +  
              pow(avatarLocation[2] - myLocation[2], 2));
}


//**************************************************************
//  Function:   setWorld
//  Purpose:    ???
//  Parameters: pointer to the world
//  Returns:    void
//**************************************************************
void kPartList::setWorld(pfNode *world)
{
 world_ = world;
}


//**************************************************************
//  Function:   CheckForIsect
//  Purpose:    checks for wand intersections with names on the 
//              list.  This allows us to use names like hot links
//  Parameters: pointer to the scene
//  Returns:    distance as a double
//**************************************************************
void kPartList::HandleNameIsect()
{
  int numNames = 0;
  pfVec3 wandPos, extension;
  pfNode *temp_new_av;
  pfString *string;
  pfText *tempText;
  pfBox tempbox;

  temp_new_av = new pfDCS();
  string = new pfString;
  tempText = new pfText;
  numNames = getNumChildren();
  cvrnPrintf("numNames is: %d\n", numNames);

  CAVEGetPosition(CAVE_WAND, wandPos.vec);
  //CAVENavConvertCAVEToWorld(wandPos.vec, wandInWorld.vec);
  
  extension.vec[0] = 0.0;
  extension.vec[1] = 0.5;
  extension.vec[2] = 0.2;

  if(numNames > 0)
    {
      for(int i = 1; i < numNames; i++)
	{
	  temp_new_av = getChild(i);
	  cvrnPrintf("child i is: %s\n", temp_new_av->getName());
	  tempText = ((pfText *)(((pfDCS *)temp_new_av)->getChild(0)));
	  string = tempText->getString(0);
	  cvrnPrintf("string is: %s\n", string->getString());
	  cvrnPrintf("wand: (%f, %f, %f)\n", wandPos.vec[0], wandPos.vec[1], wandPos.vec[2]);
	  tempbox = *string->getBBox();
	  tempbox.extendBy(extension);
	  cvrnPrintf("bbox min x: (%f)\n", tempbox.min[0]);
	  cvrnPrintf("bbox min z: (%f)\n", tempbox.min[1]);
	  cvrnPrintf("bbox min y: (%f)\n", tempbox.min[2]);
	  cvrnPrintf("bbox max x: (%f)\n", tempbox.max[0]);
	  cvrnPrintf("bbox max z: (%f)\n", tempbox.max[1]);
	  cvrnPrintf("bbox max y: (%f)\n", tempbox.max[2]);
	  if(tempbox.contains(wandPos))
	    {
	      cvrnPrintf("setting color...\n");
	      string->setColor(1.0, 0.0, 0.0, 0.0);
	    }
	  else string->setColor(0.0, 1.0, 0.0, 0.0);
	}
      return;
    }
  else return;
}
//**************************************************************
//  Function:   app
//  Purpose:    this is the programmer-defined Performer function 
//              that is automatically called each time Performer
//              traverses the scene graph
//  Parameters: pointer to the traverser
//  Returns:    void
//**************************************************************
int kPartList::app(pfTraverser *trav)
{
  int avatarNum;     // number of avatars currently tracked by 
                     // the avatar manager

  static float y_offset = Y_OFF;    //spacing between names on list
  pfVec3 myLocation, avatarPos;     // these are the two location
                                    // vectors for calculating distance

  pfDCS *avatar;     // temp storage representing a single
                     // avatar within g_limAvatarDCS

  const char *avDCSName;     // temp variable for identifying 
                             // the avataring being pointed to 
                             // by g_limAvatarDCS->getChild(j)

  long avatarID;     // avDCSName is converted to an ID number,
                     // which is required by the call to 
                     // avatarDB->find(avatarID, theAvatar)
 
  char avatarName[30];     // name written to list, *including* distance
  double avatarDistance;     // distance between you and another avatar

  CAVERNst_baseAvatar_c *theAvatar;     // pointer to an avatar in the 
                                        // in the avatar database

  // don't bother to continue if 
  // this object isn't active...
  //////////////////////////////
  if (!active_)
    return pfDCS::app(trav);
  
  // find out how many avatars are 
  // currently being tracked
  ////////////////////////////////
  avatarNum = g_limAvatarDCS->getNumChildren(); 
  
  // check for avatars before trying to list them
  ///////////////////////////////////////////////
  if (avatarNum > 0)
    {
      CAVEGetPosition(CAVE_HEAD_NAV, myLocation.vec);     // get your position
    }

  // do this loop once for each avatar
  ////////////////////////////////////
  for (int j=0; j < avatarNum; j++)
    {
      avatar = (pfDCS*)g_limAvatarDCS->getChild(j);     // get j_th child
      avDCSName = avatar->getName();     // get it's name for find function
      avatarID = atol(avDCSName);     // convert name to an ID number

      // find this avatar in the manager so we can read it's data
      ///////////////////////////////////////////////////////////
      g_limAvatarManager->avatarDB->find(avatarID, theAvatar);

      // next 4 lines set up the input for getDistance
      ////////////////////////////////////////////////
      avatarPos[0] = theAvatar->bodyX;
      avatarPos[1] = theAvatar->bodyY;
      avatarPos[2] = theAvatar->bodyZ;
      avatarDistance = getDistance(avatarPos, myLocation);

      // convert the name *and* distance to a string
      //////////////////////////////////////////////
      sprintf(avatarName, "%s %5.2f",
	      strtok(theAvatar->helloData, " "),
	      avatarDistance);
      
      //if this is the first one added to the list,
      //getNumChildren will be zero.  But if getNumChildren
      //is not 0, and is less than avatarNum, then this
      //must be a new participant.  (We have to do <= 
      //because the first child for this obj is the title)
      /////////////////////////////////////////////////////
     
      if(getNumChildren() <= avatarNum)
	{
	  new_avDCS = new pfDCS();     // create a new DCS for this list item

	  //set the name for this DCS
	  ////////////////////////////////////
	  char new_avName[10];
	  sprintf(new_avName,"Name");
	  cvrnPrintf("%s\n", new_avName);
	  new_avDCS->setName(new_avName);

	  new_avString = new pfString;     // create a string to hold words
	  new_avText = new pfText;     // create pfText to hold pfString
	  
	  new_avString->setFont(fnt);     // same font as above:  Times-Elfin
	  new_avString->setColor(0.0f, 1.0f, 0.0f, 0.0f);    // GREEN!
	  new_avString->setString(avatarName);     // set the name

	  new_avText->addString(new_avString);     // add pfString to pfText

	  new_avDCS->addChild(new_avText);     // add pfText to pfDCS
	  new_avDCS->setScale(0.65, 0.65, 0.65);     // relative to this obj!!
	  new_avDCS->setTrans(0.0, 0.0, y_offset);     // account for spacing

	  addChild(new_avDCS);     // add the pfDCS to this object

	  y_offset += Y_OFF;    // decreasing offset places the next
	                        // name below this name.
	}
      else 
	{
	  pfText *updatedText = new pfText;
	  if (j < getNumChildren()) { j++; }
	  updatedText = ((pfText *)(((pfDCS *)(getChild(j)))->getChild(0)));
	  ((pfString *)(updatedText->getString(0)))->setString(avatarName);
	}
	
    }
  return pfDCS::app(trav);
}

int kPartList::needsApp(void)
{
  return TRUE;
}


int kPartList::isActive()
{
  return active_;
}

