#include "lim_global.h++";
#include "kRadar.h++"

/************************************************************************/
/* Performer type data
*/

pfType *kRadar::classType_ = NULL; 

void kRadar::init(void)
{
 if (classType_ == NULL)
	{
        pfDCS::init();				/* Initialize the parent class */
        classType_ =  new pfType(pfDCS::getClassType(), "kRadar");	/* Create the new pfType for this class, based on the parent class's type */
	}
}
/************************************************************************/



kRadar::kRadar()
{
 pfGeode *radar;
 pfGeode *you;
 pfVec4 color;
 blips = new pfGroup;
 setType(classType_);  /* set the Performer type of this instance */
 active_ = 1;
 setScale(0.5f);
 prevTime_ = pfGetTime();
 color[0]= 0.75f;
 color[1]= 0.75f;
 color[2]= 0.75f;
 color[3]= 1.0f;
 radar = createSphere(1.0f, color,PFGS_WIREFRAME); 
 addChild(radar);
 color[0]= 1.0f;
 color[1]= 0.0f;
 color[2]= 0.0f;
 color[3]= 1.0f;
 you = createSphere(0.1f, color,PFGS_WIREFRAME); 
 addChild(you);
 addChild(blips);
}

pfGeode * kRadar::createSphere(float radius,pfVec4 color, int mode)
{
 pfGeoSet *gset;
 pfVec4 *colorAttr;
 pfMatrix matrix;
 pfGeode *geode = new pfGeode;
 active_ = 1;
 gset = pfdNewSphere(32,pfGetSharedArena());
 colorAttr = (pfVec4 *) pfMalloc(sizeof(pfVec4),pfGetSharedArena());
 *colorAttr = color;
 gset->setAttr(PFGS_COLOR4,PFGS_OVERALL,colorAttr,NULL);
 gset->setDrawMode(mode, 1);
 matrix.makeScale(radius,radius,radius);
 pfdXformGSet(gset,matrix);
 geode->addGSet(gset);
 return geode;
}

void kRadar::setWorld(pfNode *world)
{
 world_ = world;
}


int kRadar::app(pfTraverser *trav)
{
 int avatarNum, blipsNum;
 pfVec3 myLocation, difference, avatarPos;
 pfDCS *avatar;
 const char *DCSName;
 long avatarID;
 CAVERNst_baseAvatar_c *theAvatar;
 if (!active_)
	return pfDCS::app(trav);
 blipsNum =blips->getNumChildren();
 avatarNum = g_limAvatarDCS->getNumChildren(); 
 for (int i=0; i < blipsNum; i++)
 {
    blips->removeChild((pfDCS*)blips->getChild(0));
 }
 for (i=0; i < avatarNum; i++)
 {
   avatar = (pfDCS*)g_limAvatarDCS->getChild(i);
   DCSName = avatar->getName();
   avatarID = atol(DCSName);
   g_limAvatarManager->avatarDB->find(avatarID, theAvatar);
   CAVEGetPosition(CAVE_HEAD, myLocation.vec);
   avatarPos[0] = theAvatar->bodyX;
   avatarPos[1] = theAvatar->bodyY;
   avatarPos[2] = theAvatar->bodyZ - 6.0f;
   CAVENavConvertWorldToCAVE(avatarPos.vec, difference.vec);
   if ((difference[0] > -75) && (difference[0] < 75) && 
       (difference[1] > -75) && (difference[1] < 75) && 
       (difference[2] > -75) && (difference[2] < 75))   
   {
      difference[0] = difference[0]/100.0f;
      difference[1] =  difference[1]/100.0f;
      difference[2] =  difference[2]/100.0f;
      pfDCS *blipDCS;
      pfVec4 color;
      pfGeode *blipGeode;
      blipDCS = new pfDCS;
      color[0]= 0.0f;
      color[1]= 1.0f;
      color[2]= 0.0f;
      color[3]= 1.0f;
      blipGeode = createSphere(0.05f, color,PFGS_FLATSHADE); 
      blipDCS->addChild(blipGeode);
      blips->addChild(blipDCS);
      blipDCS->setTrans(difference[0], difference[1], difference[2]);
   }
 }
 return pfDCS::app(trav);
}

int kRadar::needsApp(void)
{
 return TRUE;
}


int kRadar::isActive()
{
    return active_;
}
