#include "ra.h++"
#include "lim_global.h++"
#include "lim_init.h++"
#include "lim_perf.h++"

CAVERN_irbChannel_c *g_limAudioChannel = NULL;
CAVERN_irbKey_c *g_limAudioKey=NULL;
CAVERN_irbLink_c *g_limAudioLink= NULL;

void createAudioChannel();
void makeChannelAndAudioLink(char *remoteHost);
void newAudioCB(CAVERN_irbKey_c::CAVERN_irbKeyEvent_t event,
	     CAVERN_irbKey_c *thisKey,
	     void* userData);

void limAudioInit(char ** argv)
{
	CAVERN_irb_c::status_t irbStatus;
	CAVERN_irbKeyId_c audioKeyId;

	audioKeyId.setName("AUDIO_KEY");
	g_limAudioKey = g_limPIRB->define(&audioKeyId, NULL, &irbStatus);
	g_limAudioKey->trigger(newAudioCB, (void *) NULL);

	if (g_limRemoteConnect) {
		/// Open the channel to a remote IRB specified on the command line.
		createAudioChannel();
		makeChannelAndAudioLink(argv[1]);
	}

}

void createAudioChannel()
{
	g_limAudioChannel = g_limPIRB->createChannel();
}


// Make a channel and a link to link the AUDIO_KEY
void makeChannelAndAudioLink(char *remoteHost)
{
	CAVERN_irbKeyId_c audioKeyId;
	audioKeyId.setName("AUDIO_KEY");

	CAVERN_irbChannel_c::status_t channelRetStatus;

	CAVERN_irbId_c remoteIRBId;
	remoteIRBId.setAddress(remoteHost);

	do {
		cvrnPrintf("Audio Channel: Attempting to Connect...\n");
		g_limAudioChannel->open(&remoteIRBId,NULL,CAVERN_irbChannel_c::RELIABLE,
		       &channelRetStatus);
		g_limAudioChannel->trigger(limChannelCB, (void*) remoteHost);

		sleep(2);
	} while(channelRetStatus != CAVERN_irbChannel_c::OK);

	cvrnPrintf("Audio Channel: Connected\n");

	if (g_limAudioChannel) {

		if (g_limAudioLink) delete g_limAudioLink;

		CAVERN_linkAttrib_c linkAttribute;

		linkAttribute.setInitialSynchType(CAVERN_linkAttrib_c::NONE);
		g_limAudioLink = g_limAudioChannel->link(g_limAudioKey, &audioKeyId,&linkAttribute);
		if (g_limAudioLink == NULL) {
			printf("Yo dude! The link didn't work!\n");
			fflush(stdout);
		}
	}
}

void limHandleRecording()
{
	static int recordState = 0;
	static audioRecorder *ro=NULL;
	int size;
	CAVERN_irbKey_c::status_t status;
	float wandPos[3];

	if (CAVEBUTTON3) {
		if (recordState == 0) {
			ro = new audioRecorder;
			char hname[128], outfile[128];
			gethostname(hname, 128);
			sprintf(outfile,"LIMBO_TEMP/%s_temp.aiff",hname);
			ro->init(outfile);
			recordState = 1;

		} else {
			CAVEGetPosition(CAVE_HEAD, wandPos);
			g_limMicDCS->setTrans(wandPos[0]+1, wandPos[1]+3, wandPos[2]-0.5);
			ro->record();
		}
	} else {
		if (recordState) {
			g_limMicDCS->setTrans(0,-10,-2);

			ro->stop();
			char hname[128], outfile[128];
			gethostname(hname, 128);
			sprintf(outfile,"LIMBO_TEMP/%s_temp.aiff",hname);
			g_limAudioKey->import(outfile,&size, &status);
			delete ro;
			recordState = 0;
		}
	}
}


// Callback for when a new audio arrives.
void newAudioCB(CAVERN_irbKey_c::CAVERN_irbKeyEvent_t event,
	     CAVERN_irbKey_c *thisKey,
	     void* userData)
{
	int size;

	CAVERN_irbKey_c::status_t status;
	thisKey->getSize(&size, &status);

	char hname[128], outfile[128];
	gethostname(hname, 128);
	sprintf(outfile,"%s.aiff",hname);
	char command[128];

	if (size) {
		cvrnPrintf("AUDIO DATA ARRIVED. EXPORTING\n");
		thisKey->export(outfile, &status);
		cvrnPrintf("EXPORTING DONE\n");
		sprintf(command,"playaiff %s&",outfile);
		system(command);
	}
}

