#include "lim_global.h++"
#include "lim_init.h++"
#include "lim_perf.h++"
#include  "lim_bullet.h++"
#include  "kBullet.h++"

CAVERN_irbChannel_c *g_limBulletChannel = NULL;
CAVERN_irbKey_c *g_limBulletKey=NULL;
CAVERN_irbLink_c *g_limBulletLink= NULL;
char bulletBuff[64];

void createBulletChannel();
void makeChannelAndBulletLink(char *remoteHost);
void newBulletCB(CAVERN_irbKey_c::CAVERN_irbKeyEvent_t event,
	     CAVERN_irbKey_c *thisKey,
	     void* userData);

void limBulletInit(char ** argv)
{
	CAVERN_irb_c::status_t irbStatus;
	CAVERN_irbKeyId_c bulletKeyId;

	bulletKeyId.setName("BULLET_KEY");
	g_limBulletKey = g_limPIRB->define(&bulletKeyId, NULL, &irbStatus);
	g_limBulletKey->trigger(newBulletCB, (char *) bulletBuff);

	if (g_limRemoteConnect) {
		/// Open the channel to a remote IRB specified on the command line.
		createBulletChannel();
		makeChannelAndBulletLink(argv[1]);
	}

}

void createBulletChannel()
{
	g_limBulletChannel = g_limPIRB->createChannel();
}


// Make a channel and a link to link the AUDIO_KEY
void makeChannelAndBulletLink(char *remoteHost)
{
	CAVERN_irbKeyId_c bulletKeyId;
	bulletKeyId.setName("BULLET_KEY");

	CAVERN_irbChannel_c::status_t channelRetStatus;

	CAVERN_irbId_c remoteIRBId;
	remoteIRBId.setAddress(remoteHost);

	do {
		cvrnPrintf("Bullet Channel: Attempting to Connect...\n");
		g_limBulletChannel->open(&remoteIRBId,NULL,CAVERN_irbChannel_c::RELIABLE,
		       &channelRetStatus);
		g_limBulletChannel->trigger(limChannelCB, (void*) remoteHost);

		sleep(2);
	} while(channelRetStatus != CAVERN_irbChannel_c::OK);

	cvrnPrintf("Bullet Channel: Connected\n");

	if (g_limBulletChannel) {

		if (g_limBulletLink) delete g_limBulletLink;

		CAVERN_linkAttrib_c linkAttribute;

		linkAttribute.setInitialSynchType(CAVERN_linkAttrib_c::NONE);
		g_limBulletLink = g_limBulletChannel->link(g_limBulletKey, &bulletKeyId,&linkAttribute);
		if (g_limBulletLink == NULL) {
			printf("Yo dude! The link didn't work!\n");
			fflush(stdout);
		}
	}
}


// Callback for when a new audio arrives.
void newBulletCB(CAVERN_irbKey_c::CAVERN_irbKeyEvent_t event,
	     CAVERN_irbKey_c *thisKey,
	     void* userData)
{
	int size;
	CAVERN_irbKey_c::status_t status;
	CAVERNplus_datapack_c datapack;
        pfVec3 position;
        pfVec3 velocity;
        pfVec3 caveposition;
        pfVec3 cavevelocity;
        char *data;
	thisKey->getAutoAlloc(data, &size, &status);
	datapack.setBuffer(data, size);
	position.vec[0] = datapack.unpackFloat();
	position.vec[1] = datapack.unpackFloat();
	position.vec[2] = datapack.unpackFloat();
	velocity.vec[0] = datapack.unpackFloat();
	velocity.vec[1] = datapack.unpackFloat();
	velocity.vec[2] = datapack.unpackFloat();

        CAVENavConvertWorldToCAVE(position.vec, caveposition.vec);
        CAVENavConvertVectorWorldToCAVE(velocity.vec, cavevelocity.vec);
        pfDCS *newBox;


        newBox = new pfDCS();
        pfGeoSet *ball;
        pfGeode *geode;
        newBox->setTrans(caveposition[0],caveposition[1]+.5,caveposition[2]);
        ball = pfdNewCube(pfGetSharedArena());
        geode = new pfGeode;
        geode->addGSet(ball);
        newBox->addChild(geode);
        g_limBullet->addChild(newBox);
/*
        kBullet *newkBullet;
        newkBullet = new kBullet(caveposition, cavevelocity, 0);
        newkBullet->setWorld(g_limBadBullet);
        g_limBullet->addChild(newkBullet);
*/
}

void packAndSendBulletData( float bulletX, float bulletY, 
                                       float bulletZ, float vectX,
				       float vectY, float vectZ)
{

	CAVERNplus_datapack_c *datapack;
	CAVERN_irbKey_c::status_t keyStatus;
        datapack = new CAVERNplus_datapack_c(bulletBuff, 64);

	datapack->setBuffer(bulletBuff, 64);
	datapack->packFloat(bulletX);
	datapack->packFloat(bulletY);
	datapack->packFloat(bulletZ);
	datapack->packFloat(vectX);
	datapack->packFloat(vectY);
	datapack->packFloat(vectZ);

	int size = 64;
	g_limBulletKey->put((char*) &bulletBuff,&size, &keyStatus);
}

