#include "lim_global.h++"
#include "lim_perf.h++"
#include "lim_main.h++"
#include "lim_init.h++"
#include "lim_model.h++"

CAVERN_irbChannel_c *g_limModelChannel = NULL;
CAVERN_irbChannel_c *g_limModelMetaChannel = NULL;
CAVERN_irbLink_c *g_limModelLink[_LIM_MAX_MODELS];
CAVERN_irbLink_c *g_limModelMetaLink[_LIM_MAX_MODELS];
CAVERN_irbKey_c *g_limListOfModelMetaDataKeys[_LIM_MAX_MODELS];
CAVERN_irbKey_c *g_limListOfModelDataKeys[_LIM_MAX_MODELS];
pfDCS *g_listOfModelDCS[_LIM_MAX_MODELS];
int g_replacementFlag[_LIM_MAX_MODELS];

int g_limNeedToDoLoad = 0;
pfDCS *g_dcsReplacement = NULL;

void newModelCB(CAVERN_irbKey_c::CAVERN_irbKeyEvent_t event,
	     CAVERN_irbKey_c *thisKey,
	     void* userData);

void newMetaDataCB(CAVERN_irbKey_c::CAVERN_irbKeyEvent_t event,
	     CAVERN_irbKey_c *thisKey,
	     void* userData);

void openModelChannels(char *remoteHost);
void makeModelLinks(int index);

void limModelInit(char **argv)
{
	for(int i = 0; i < _LIM_MAX_MODELS; i++) g_limListOfModelMetaDataKeys[i] = NULL;

	CAVERN_irbKeyId_c modelKeyId, modelMetaKeyId;
	CAVERN_irb_c::status_t irbStatus;

	if (g_limRemoteConnect) {
		g_limModelChannel = g_limPIRB->createChannel();
		g_limModelMetaChannel = g_limPIRB->createChannel();
		openModelChannels(argv[1]);
	}


	for (i = 0; i < _LIM_MAX_MODELS; i++) {

		g_replacementFlag[i] = 0;

		char str[1024];
		sprintf(str,"Mo%d",i);
		modelKeyId.setName(str);
		g_limListOfModelDataKeys[i] = g_limPIRB->define(&modelKeyId, NULL, &irbStatus);

		// If there is data already in the key then load it as a model.
		int size;
		CAVERN_irbKey_c::status_t status;
		g_limListOfModelDataKeys[i]->getSize(&size, &status);

		newModelCB(CAVERN_irbKey_c::NEW_DATA_ARRIVED, g_limListOfModelDataKeys[i],
			   (void *) new int(i));
			   
		sprintf(str,"Me%d",i);
		modelMetaKeyId.setName(str);
		g_limListOfModelMetaDataKeys[i] = g_limPIRB->define(&modelMetaKeyId, NULL, &irbStatus);

		newMetaDataCB(CAVERN_irbKey_c::NEW_DATA_ARRIVED, g_limListOfModelMetaDataKeys[i],
			      (void *) g_listOfModelDCS[i]);

		g_limListOfModelDataKeys[i]->trigger(newModelCB, (void *) new int(i));
		g_limListOfModelMetaDataKeys[i]->trigger(newMetaDataCB, (void *) g_listOfModelDCS[i]);

		if (g_limRemoteConnect) {
			makeModelLinks(i);
		}

	}


}

void limSaveModelMetaDataStates()
{
	CAVERN_irbKey_c::status_t status;
	CAVERN_irbLink_c::status_t statusL;

	for (int i = 0; i < _LIM_MAX_MODELS; i++) {
		g_limListOfModelMetaDataKeys[i]->commit(&status);
		if (g_limRemoteConnect)
			g_limModelMetaLink[i]->commitRemote(&statusL);
	}
}

void openModelChannels(char *remoteHost)
{
	CAVERN_irbChannel_c::status_t channelRetStatus;

	CAVERN_irbId_c remoteIRBId;
	remoteIRBId.setAddress(remoteHost);

	do {
		cvrnPrintf("Model Channel: Attempting to Connect...\n");
		g_limModelChannel->open(&remoteIRBId,NULL,CAVERN_irbChannel_c::RELIABLE,
		       &channelRetStatus);
		//		g_limModelChannel->trigger(limChannelCB, (void*) remoteHost);

		sleep(2);
	} while(channelRetStatus != CAVERN_irbChannel_c::OK);
	cvrnPrintf("Model Channel: Connected\n");

	do {
		cvrnPrintf("Model Meta Channel: Attempting to Connect...\n");
		g_limModelMetaChannel->open(&remoteIRBId,NULL,CAVERN_irbChannel_c::UNRELIABLE,
		       &channelRetStatus);
		//		g_limModelChannel->trigger(limChannelCB, (void*) remoteHost);

		sleep(2);
	} while(channelRetStatus != CAVERN_irbChannel_c::OK);
	cvrnPrintf("Model Meta Channel: Connected\n");

}

// Make a link to link the model and model meta keys
void makeModelLinks(int index)
{
	CAVERN_irbKeyId_c modelKeyId;
	CAVERN_irbKeyId_c modelMetaKeyId;

	char str[1024];
	sprintf(str,"Mo%d",index);
	modelKeyId.setName(str);

	sprintf(str,"Me%d",index);
	modelMetaKeyId.setName(str);

	if (g_limModelChannel) {

		/// Create a link to link the local key named MODEL_KEY with the remote key
		/// also named MODEL_KEY.
		CAVERN_linkAttrib_c linkAttribute;
		//		linkAttribute.setInitialSynchType(CAVERN_linkAttrib_c::NONE);

		g_limModelLink[index] = g_limModelChannel->link(g_limListOfModelDataKeys[index], &modelKeyId,&linkAttribute);
		if (g_limModelLink[index] == NULL) {
			cvrnPrintf("Yo dude! The link didn't work!\n");
		}
	}

	if (g_limModelMetaChannel) {

		/// Create a link to link the local key named MODEL_KEY with the remote key
		/// also named MODEL_KEY.
		CAVERN_linkAttrib_c linkAttribute;
		//		linkAttribute.setInitialSynchType(CAVERN_linkAttrib_c::NONE);

		g_limModelMetaLink[index] = g_limModelChannel->link(g_limListOfModelMetaDataKeys[index], &modelMetaKeyId,&linkAttribute);
		if (g_limModelMetaLink[index] == NULL) {
			cvrnPrintf("Yo dude! The model meta link didn't work!\n");
		}
	}

}


// Callback for when a new model arrives.
void newModelCB(CAVERN_irbKey_c::CAVERN_irbKeyEvent_t event,
	     CAVERN_irbKey_c *thisKey,
	     void* userData)
{

	int *index = (int *) userData;

	int size;
	CAVERN_irbKey_c::status_t status;
	thisKey->getSize(&size, &status);
	if (size) {

		char hname[128], outfile[128];
		gethostname(hname, 128);
		sprintf(outfile,"LIMBO_TEMP/%stemp%d.iv",hname, *index);

		thisKey->export(outfile, &status);

		thisKey->commit(&status);

		// Ordinarily I would do a performer load right
		// after the export but it seems something about Performer
		// really barfs. So it works on an O2 but not an Onyx2.
		// So instead I signal that the model has been exported
		// and let the main loop do the loading.
		g_replacementFlag[*index] = 1;
	} else {
		thisKey->commit(&status);
		g_replacementFlag[*index] = -1;
	}


}

// New incoming object position data arrives.
void newMetaDataCB(CAVERN_irbKey_c::CAVERN_irbKeyEvent_t event,
	     CAVERN_irbKey_c *thisKey,
	     void* userData)
{
	float vec[3];
	int size;
	CAVERN_irbKey_c::status_t status;
	thisKey->getSize(&size, &status);
	if (size == (sizeof(float) * 3)) {

		// Pull out the position data.
		thisKey->get((char*) vec, &size, &status);

		// Apply this to the dcs.
		
		pfDCS *aDCS = (pfDCS*) userData;
		aDCS->setTrans(vec[0], vec[1], vec[2]);
	}
}


void limCheckForNewObjectsToLoad(){

	for (int i = 0; i < _LIM_MAX_MODELS; i++) {
		if (g_replacementFlag[i] == 1) {

			cvrnPrintf("PERFORMER LOAD %d\n",i);
			char hname[128], outfile[128];
			gethostname(hname, 128);
			sprintf(outfile,"LIMBO_TEMP/%stemp%d.iv",hname,i);

			limPerfReplaceObject(outfile, g_listOfModelDCS[i]);

			g_replacementFlag[i] = 0;

			//limNeedToDoLoad = 0;
			cvrnPrintf("PERFORMER DONE LOAD\n");

			system("playaiff LIMBO_SOUNDS/loadsound.aif&");
		} else if (g_replacementFlag[i] == -1) {
			limPerfRemoveObject(g_listOfModelDCS[i]);
			g_replacementFlag[i] = 0;
		}
	}
}
