#include "lim_global.h++"
#include "lim_perf.h++"
#include "lim_model.h++"
#include "kFlatRadar.h++"
#include "kPartList.h++"
#include "teleport.h++"

pfGroup *g_limBullet = NULL;
pfGroup *g_limBadBullet = NULL;
pfScene *g_limScene = NULL;
pfDCS *g_arrowDCS = NULL;
pfDCS *g_limTextDCS = NULL;
pfDCS *g_limMainGroup = NULL;
pfDCS *g_limWandDCS = NULL;
pfDCS *g_limMicDCS = NULL;
pfDCS *g_sphereDCS = NULL;
pfDCS *g_modelDCS = NULL;
pfDCS *g_selectedDCS = NULL;
pfDCS *g_limFlatRadarDCS = NULL;
kPartList *g_limPartList = NULL;
kFlatRadar *g_limFlatRadar = NULL;
int g_limObjectCount = 0;

pfDCS * createScene(pfChannel *chan, pfScene *scene);
void loadMainSceneObjects(pfDCS *parent);
pfDCS * loadMicObject();

void limPerformerInit(int *argc, char *** argv)
{

	CAVEFar = 100000;
	// Initialize Performer
	pfInit();

	// Equivalent to CAVE Config. Must be called after pfInit()
	pfCAVEConfig(argc,*argv,NULL);

	// Performer config
         	pfConfig();

	// Craetes all the channels for performer
	pfCAVEInitChannels();

	// Create a scene composed of objects loaded from filenames specified
	// in the command line arguments.

	g_limScene = new pfScene;
	g_limMainGroup = createScene(pfCAVEMasterChan(),g_limScene);

}

// Perform one frame update
void limPerfOneCycle()
{
		// Required by performer to synch things to a certain frame rate.
      		pfSync();

		// Updates tracker data and sets up new projects for each channel.
		pfCAVEPreFrame();

		// Required by performer to trigger cull and draw
		pfFrame();

		// Updates non latency critical data.
		pfCAVEPostFrame();

}

pfDCS * createScene(pfChannel *chan, pfScene *scene)
{
	/* Create a pfScene */
	pfGeoState *gstate = new pfGeoState;
	pfDCS *group = new pfDCS;

	/* Set up the default GeoState to specify the how the geometry is to
	   be displayed.
	   */
	gstate->setMode(PFSTATE_ENLIGHTING, PF_ON);
	gstate->setMode(PFSTATE_CULLFACE, PFCF_OFF);
	scene->setGState(gstate);

	/* Add a light source to the scene as well as the dcs */
	pfLightSource *myLightSource = new pfLightSource;
	myLightSource->setPos(10,-10,10,1);

	pfLightSource *myLightSource2 = new pfLightSource;
	myLightSource2->setPos(-10,-10,10,1);


	scene->addChild(myLightSource);
	scene->addChild(myLightSource2);


	/* Add the group, and load the objects under it */
	scene->addChild(group);
	scene->addChild(loadMicObject());

        g_limBullet = new pfGroup;
        g_limBadBullet = new pfGroup;
        group->addChild(g_limBullet);
        group->addChild(g_limBadBullet);
	
	kPartList::init();
        g_limTextDCS = new pfDCS;
	g_limPartList = new kPartList();
	g_limTextDCS->addChild(g_limPartList);
	scene->addChild(g_limTextDCS);

	loadMainSceneObjects(group);

	kFlatRadar::init();
        g_limFlatRadarDCS = new pfDCS;
        g_limFlatRadar = new kFlatRadar();
        
        g_limFlatRadarDCS->addChild(g_limFlatRadar);
        g_limFlatRadarDCS->addChild(g_limPartList);
        g_limFlatRadar->setRot(0.0f, 90.0f, 0.0f);
        scene->addChild(g_limFlatRadarDCS);

	void loadManipulableDummyModels(pfDCS *parent);
	loadManipulableDummyModels(group);
	
	/******TELEPORT CODE***********************/
	/*
	pfVec3 enter, ex, location;
	pfBox bounds;
	pfBox bounds2;

	enter[0] = 20;     // x
	enter[1] = 20;    // z
	enter[2] = 0;      // y

	ex[0] = 50;        // x
	ex[1] = 50;        // z
	ex[2] = 0;       // y

	bounds.min[0] = enter[0];      // x
	bounds.max[0] = enter[0]+20;      // x
	bounds.min[1] = enter[1];       // z
	bounds.max[1] = enter[1]+20;      // z
	bounds.min[2] = enter[2];       // y
	bounds.max[2] = enter[2]+20;      // y

	bounds2.min[0] = ex[0];      // x
	bounds2.max[0] = ex[0]+20;      // x
	bounds2.min[1] = ex[1];       // z
	bounds2.max[1] = ex[1]+20;      // z
	bounds2.min[2] = ex[2];       // y
	bounds2.max[2] = ex[2]+20;      // y
	
	
	g_limTeleportDCS = new pfDCS;
	Teleport::init();
	
	Teleport *port1 = new Teleport(enter, ex, bounds, 1);
	Teleport *port2 = new Teleport(ex, enter, bounds2, 2);
	
	cvrnPrintf("new port successful\n");
	
	port1->DrawTeleport(g_limTeleportDCS);
	port2->DrawTeleport(g_limTeleportDCS);
	
	g_limTeleportDCS->addChild(port1);
	g_limTeleportDCS->addChild(port2);

	
	
        group->addChild(g_limTeleportDCS);
	*/
	/**********************END TELEPORT*******************/


	/* Assign the scene to the display channel */
	chan->setScene(scene);

	return group;
}

void loadManipulableDummyModels(pfDCS *parent)
{
	for (int i = 0; i < _LIM_MAX_MODELS; i++) {
		pfDCS *newDCS = new pfDCS;
		char objName[256];
		sprintf(objName,"Jason %d", i);
		newDCS->setName(objName);

		newDCS->setTravMask(PFTRAV_ISECT,
				  // Intersection mask
				  0x8,
				  PFTRAV_SELF | PFTRAV_DESCEND | PFTRAV_IS_CACHE,
				  PF_SET);
		g_listOfModelDCS[i] = newDCS;
		parent->addChild(newDCS);
	}		
}

void limPerfRemoveObject(pfDCS *parent)
{
	if (parent) {
		parent->setTrans(0,0,0);
		if (parent->getNumChildren()) {
			pfNode *aChild = parent->getChild(0);
			if (aChild)
				parent->removeChild(aChild);
		}


	}
}

void limPerfReplaceObject(char *filename,pfDCS *parent)
{
	pfNode *obj;

	if (obj = pfdLoadFile(filename)) {
		if (parent) {
			int num = parent->getNumChildren();
			fflush(stdout);
			if (num) {
				pfNode *aChild = parent->getChild(0);
				if (aChild) {
					parent->removeChild(aChild);
					parent->addChild(obj);
				}
			} else {
				parent->addChild(obj);
			}
		}
	}
}

pfDCS * loadMicObject()
{

	pfNode *obj;

	if (obj = pfdLoadFile("LIMBO_MODELS/mic.iv")) {

		g_limMicDCS = new pfDCS;
		g_limMicDCS->setTrans(0,-10,50000);
		g_limMicDCS->addChild(obj);
		return g_limMicDCS;
	}
	return NULL;
}

void loadMainSceneObjects(pfDCS *parent)
{
	pfNode *obj;


	if (obj = pfdLoadFile("amyoffice.pfb")) {
        	g_modelDCS = new pfDCS;
		g_modelDCS->setTrans(50, 15, 0.5);
		g_modelDCS->addChild(obj);
		parent->addChild(g_modelDCS);
	}
	if (obj = pfdLoadFile("LIMBO_MODELS/twirl.iv")) {
		parent->addChild(obj);
	}
	if (obj = pfdLoadFile("LIMBO_MODELS/sphere.iv")) {
              	g_sphereDCS = new pfDCS;
		g_sphereDCS->setTrans(0,-10,-2);
		g_sphereDCS->addChild(obj);
		parent->addChild(g_sphereDCS);
	}

	if (obj = pfdLoadFile("LIMBO_MODELS/arrow.iv")) {
              	g_arrowDCS = new pfDCS;
		g_arrowDCS->setTrans(0,-10,-2);
		g_arrowDCS->addChild(obj);
		parent->addChild(g_arrowDCS);
	}

	if (obj = pfdLoadFile("LIMBO_MODELS/box.iv")) {
              	g_selectedDCS = new pfDCS;
		g_selectedDCS->setTrans(0,-10,-2);
		g_selectedDCS->addChild(obj);
		parent->addChild(g_selectedDCS);
	}


	if (obj = pfdLoadFile("LIMBO_MODELS/limbo.iv")) {
              	pfDCS *limboDCS = new pfDCS;
		limboDCS->addChild(obj);
		parent->addChild(limboDCS);
	}

	/*
	if (obj = pfdLoadFile("LIMBO_MODELS/mic.iv")) {

		g_limMicDCS = new pfDCS;
		g_limMicDCS->setTrans(0,-10,-2);
		g_limMicDCS->addChild(obj);
		parent->addChild(g_limMicDCS);
	}
	*/

	if (obj = pfdLoadFile("LIMBO_MODELS/hand.iv")) {
		g_limWandDCS = new pfDCS;
		g_limWandDCS->setTravMask(PFTRAV_ISECT,
				  // Intersection mask
				  0x0,
				  PFTRAV_SELF | PFTRAV_DESCEND | PFTRAV_IS_CACHE,
				  PF_OR);

		g_limWandDCS->addChild(obj);

		parent->addChild(g_limWandDCS);
	}
}


// Draw the local user's Aware control.
void limUpdateAware()
{
        float headP[3], headO[3];

        //pfCAVEGetPosition(CAVE_HEAD_NAV, headP);
        CAVEGetPosition(CAVE_HEAD, headP);
//      pfCAVEGetOrientation(CAVE_HEAD,headO);
        g_limFlatRadarDCS->setTrans(headP[0]-1.5f, headP[1]+1.0f, headP[2]+0.5f);
        //g_limAwareDCS->setRot(headO[2],headO[0],headO[1]);

}

void limUpdateWand()
{
	float wandP[3], wandO[3];

	pfCAVEGetPosition(CAVE_WAND_NAV, wandP);
	pfCAVEGetOrientation(CAVE_WAND_NAV,wandO);
	g_limWandDCS->setTrans(wandP[0], wandP[1], wandP[2]);
	g_limWandDCS->setRot(wandO[2],wandO[0],wandO[1]);

}
