//*********************************************************************
// FILENAME:  teleport.h++
// AUTHOR:    Kevin Curry
// PURPOSE:   header for teleport class.  For now, portals are drawn to
//            look like the limbo logo.  Doors are 7 feet tall, but
//            the only part you see is the logo at your feet.
//*********************************************************************


#ifndef _TELEPORT_H++
#define _TELEPORT_H++

#include <Performer/pfdu.h>
#include <Performer/pf/pfGeode.h>
#include <Performer/pf/pfTraverser.h>
#include <Performer/pf/pfSCS.h>
#include <Performer/pr/pfLinMath.h>
#include <pfcave.h>
#include <Performer/pr/pfGeoMath.h> 

extern pfDCS *g_limTeleportDCS;

class Teleport : public pfDCS
{
private:
  static pfType* classType_;

  pfVec3 door_position;     // this is where the portal is in the world

  pfVec3 exit_position;     /* this is where you reappear in the world
			     * each door has a fixed exit location
			     */
  pfBox port_bounds;        /* pfBoxes are used for isect testing
                             * this box tests if an avatar enters a portal
                             */

  int id_number;

  /**Need private data members here for drawing door.  Can this be in GL?!?**/

public:
  static void init();
  Teleport();                           // Constructor
  Teleport(pfVec3 init_pos);            // constructs at specified location
  Teleport(pfVec3 init_pos,             // constructs with specified locations
           pfVec3 init_exit,
	   pfBox &init_bounds, int id     );
  ~Teleport();                          // Destructor

  void DrawTeleport(pfDCS* parent);     // this is called by the constructors
  //  void DrawTeleport();

  int IsInBox(const pfVec3 av_loc);     // checks to see if avatar is in portal
  
  pfVec3 GetPosition();                 // return door_position
  pfVec3 GetExit();                     // return exit_position

  void PortObject(/*pfDCS *av*/);     // handles the actual teleportation

  static pfType* getClassType(void){ return classType_; };
  virtual int needsApp(void);
  virtual int app(pfTraverser*);
};

#endif
